﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Web;
using Microsoft.AccessControl2.SDK;

namespace Azure.AccessControl.SecurityModule
{
    internal class SWTModule : IHttpModule
    {
        //service namesapce setup in azure
        string _serviceNamespace = ConfigurationManager.AppSettings["serviceNamespace"];

        //azure access control service name...
        string _acsHostName = ConfigurationManager.AppSettings["acsHostname"];

        //the key for which the service was signed...
        string _trustedSigningKey = ConfigurationManager.AppSettings["trustedSigningKey"];

        //the URL that was setup in the rely party in azure
        string _trustedAudience = ConfigurationManager.AppSettings["trustedAudience"];

        //setting to allow the metadata to be shown
        bool _enableMetaData = Convert.ToBoolean(ConfigurationManager.AppSettings["enableMetadata"]);

        //setting to disable or enable the security module
        bool _disableSecurity = Convert.ToBoolean(ConfigurationManager.AppSettings["disableSecurity"]);

        const string _metaData = "$metadata";

        void IHttpModule.Dispose()
        {
        }

        void IHttpModule.Init(HttpApplication context)
        {
            context.AuthenticateRequest += new EventHandler(context_BeginRequest);
        }

        private void context_BeginRequest(object sender, EventArgs e)
        {
            if (!_disableSecurity)
            {
                string tempAcceptableURLs = String.Empty;

                //check if the audiencename has trailing slash
                tempAcceptableURLs = _trustedAudience.ToLower();
                if (tempAcceptableURLs.Substring(_trustedAudience.Length - 1, 1) == "/")
                {
                    tempAcceptableURLs = tempAcceptableURLs.Substring(0, _trustedAudience.Length - 1);
                }

                //first check if the person is requesting the wsdl or .svc
                if (_enableMetaData != false && HttpContext.Current.Request.Url.AbsoluteUri.ToLower() != tempAcceptableURLs
                        && HttpContext.Current.Request.Url.AbsoluteUri.ToLower() != tempAcceptableURLs + _metaData
                    && HttpContext.Current.Request.Url.AbsoluteUri.ToLower() != tempAcceptableURLs + "/"
                        && HttpContext.Current.Request.Url.AbsoluteUri.ToLower() != tempAcceptableURLs + "/" + _metaData
                    )
                {
                    // get the authorization header
                    string headerValue = HttpContext.Current.Request.Headers.Get("Authorization");

                    // check that a value is there
                    if (string.IsNullOrEmpty(headerValue))
                    {
                        throw new ApplicationException("unauthorized-1.1");
                    }

                    // check that it starts with 'WRAP'
                    if (!headerValue.StartsWith("WRAP "))
                    {
                        throw new ApplicationException("unauthorized-1.2");
                    }

                    string[] nameValuePair = headerValue.Substring("WRAP ".Length).Split(new char[] { '=' }, 2);

                    if (nameValuePair.Length != 2 ||
                        nameValuePair[0] != "access_token" ||
                        !nameValuePair[1].StartsWith("\"") ||
                        !nameValuePair[1].EndsWith("\""))
                    {
                        throw new ApplicationException("unauthorized-1.3");
                    }

                    // trim off the leading and trailing double-quotes
                    string token = nameValuePair[1].Substring(1, nameValuePair[1].Length - 2);

                    // create a token validator
                    TokenValidator validator = new TokenValidator(
                        this._acsHostName,
                        this._serviceNamespace,
                        this._trustedAudience,
                        this._trustedSigningKey);

                    // validate the token
                    if (!validator.Validate(token))
                    {
                        Dictionary<string, string> tokenValues = validator.GetNameValues(token);

                        string audienceValue;
                        tokenValues.TryGetValue(this._trustedAudience, out audienceValue);
                        throw new ApplicationException("unauthorized-1.4: " + _acsHostName + " - _trustedAudience: " + _trustedAudience + " -audienceValue: " + audienceValue + " - Token: " + HttpUtility.UrlDecode(token));
                    }
                }
            }
        }
    }
}