﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Data.Services.Client;
using System.Linq;
using System.Net;
using System.Text;
using System.Web;
using System.Web.Mvc;

namespace SecureoDataTestBench.Controllers
{
    public class HomeController : Controller
    {
        static string _accessControlHostName = ConfigurationManager.AppSettings.Get("AccessControlHostName");
        static string _accessControlNamespace = ConfigurationManager.AppSettings.Get("AccessControlNamespace");
        static string _rootURL = ConfigurationManager.AppSettings.Get("ServiceURL");

        //token to be placed in the request header for requests
        string _token = String.Empty;

        /// <summary>
        /// To handle the get request for the Azure Secure oData Service
        /// </summary>
        /// <returns>Action Result - View</returns>
        public ActionResult Index()
        {
            ViewBag.Message = "Welcome to the Azure Secure oData Security Test Bench";

            //first thing first we need to obtain a token from Azure...
            _token = GetTokenFromACS(_rootURL + "NorthwindService.svc");

            //now that we have a valid token we can call the service as needed...
            Uri uri = new Uri(_rootURL + "NorthwindService.svc/");
            try
            {
                var northwindEntities = new oDataServiceClient.NorthwindEntities(uri);

                //add the event handler to send the token in our request header
                northwindEntities.SendingRequest += new EventHandler<SendingRequestEventArgs>(OnSendingRequest);

                //add new data...
                var category = oDataServiceClient.Category.CreateCategory(0, "New cateogory");
                northwindEntities.AddToCategories(category);
                northwindEntities.SaveChanges();

                //retrieve the data...
                //sameple selecting data out...
                var categoiesFound = from categories in northwindEntities.Categories
                                     select categories;

                ViewBag.Categories = categoiesFound;
            }
            catch (DataServiceRequestException e)
            {
                // trap any data service exceptions such as a security error
                //in the event that the security does not allow an insert you will receive a forbidden return
                //....
            }

            return View();
        }

        /// <summary>
        /// An intercept on the request that adds in the token to the request header
        /// to allow authorization into the oData WCF Service
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnSendingRequest(object sender, SendingRequestEventArgs e)
        {
            // Add an Authorization header that contains an OAuth WRAP access token to the request.
            e.RequestHeaders.Add("Authorization", "WRAP access_token=\"" + _token + "\"");
        }

        /// <summary>
        /// Retrieve Token from Azure's Access Control Service
        /// </summary>
        /// <param name="scope">scope or audience that is seutp in Azure</param>
        /// <returns>string - token from Azure's ACS</returns>
        private static string GetTokenFromACS(string scope)
        {
            //string wrapPassword = "[YourPassword]";  // ConfigurationManager.AppSettings.Get("WrapPassword");
            //string wrapUsername = "[YourUserName]";  //ConfigurationManager.AppSettings.Get("WrapUsername");
            string wrapPassword = ConfigurationManager.AppSettings.Get("WrapPassword");
            string wrapUsername = ConfigurationManager.AppSettings.Get("WrapUsername");
            string response = String.Empty;

            // request a token from ACS
            using (WebClient client = new WebClient())
            {
                client.BaseAddress = string.Format("https://{0}.{1}", _accessControlNamespace, _accessControlHostName);

                NameValueCollection values = new NameValueCollection();
                values.Add("wrap_name", wrapUsername);
                values.Add("wrap_password", wrapPassword);
                values.Add("wrap_scope", scope);

                byte[] responseBytes = client.UploadValues("WRAPv0.9/", "POST", values);

                response = Encoding.UTF8.GetString(responseBytes);

                Console.WriteLine("\nreceived token from ACS: {0}\n", response);
            }

            return HttpUtility.UrlDecode(
                response
                .Split('&')
                .Single(value => value.StartsWith("wrap_access_token=", StringComparison.OrdinalIgnoreCase))
                .Split('=')[1]);
        }

        /// <summary>
        /// To process the Index Request for the AJAX Sample
        /// </summary>
        /// <remarks>Note: For this example we first must retrieve the token that is then pass
        /// along to the client-side script that allows us to call the service via AJAX</remarks>
        /// <returns>ActionResult - View</returns>
        public ActionResult IndexAJAX()
        {
            ViewBag.Message = "Welcome to the Azure Secure oData AJAX Security Test Bench";

            //first thing first we need to obtain a token from Azure...
            ViewBag.token = GetTokenFromACS(_rootURL + "NorthwindService.svc");

            return View();
        }
    }
}